/*********************************************************************
 *                                                                   *
 *                          RNAstrand.c                              *
 *                                                                   *
 *   Frontend of strand prediction of a structured RNA candidate.    *
 *                                                                   *
 *                c Kristin Reiche                                   *
 * $Id: RNAstrand.c,v 1.19 2007/05/29 10:18:52 kristin Exp $                                                            *
 *********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <ctype.h>
#include <unistd.h>
#include <string.h>
#include <dirent.h>

#include "fold.h"
#include "fold_vars.h"
#include "utils.h"
#include "pair_mat.h"
#include "alifold.h"
#include "zscore.h"
#include "svm.h"
#include "svm_helper.h"
#include "strand.h"

#include "RNAstrand.h"


#define MAX_NUM_NAMES 500

/* global variables */
struct svm_node *x;			// pointer auf ein svm_node
int max_attr = 64;	
struct svm_model *model;	// pointer auf ein svm_model
int predict_probability = 0;
float cutoff = 0.0001;
char versionString[1024]="2.0.0";

/* internal stuff */
int printCode=0;  /* Print map of codes to internal representation? */

/* main program 
	loads in -,out- and modelfile
	predicts infile with modelfile to outfile */
int main(int argc, char **argv) {
	char *modelDir=NULL;              /* Directory with model files */
	struct svm_model* decision_model=NULL; /* SVM classification model */
	
	/* Command line options */
	int showVersion=0; /* Shows version and exits */
	int showHelp=0;    /* Show short help and exits */
	int from=-1, from_rev=-1;       /* Scan slice from-to  */
	int to=-1, to_rev=-1;
	int doClassification=1;	/* if classification should be done */
	int i,j,r;
	int vectorOnly=0;
        int code = CODE_SCI_Z_MEANMFE_CONSMFE;      /* Which classification variable to use? Best descriptors are sci, z, meanMFE and consMFE. 
						       Corresponding model must be in RNASTRANDDIR/models/strand.model	*/
	int mafFile=0;
		
	FILE *clust_file=NULL, *clust_file_rev=NULL; /* Input file */
	char input[1024], input_rev[1024];
	
	/* Arrays storing sequences/names of the alignments */
	char *AS[MAX_NUM_NAMES], *AS_rev[MAX_NUM_NAMES];     
	char *names[MAX_NUM_NAMES], *names_rev[MAX_NUM_NAMES];
	char *window[MAX_NUM_NAMES], *window_rev[MAX_NUM_NAMES];
	
	int n_seq, n_seq_rev;  /* number of input sequences */
	int length, length_rev; /* length of alignment/window */
	int totallength;
	int trainType=0; /* specifies for -c option of which type (positive or negative example) the current trainingset is */
	
	double decValue=0, prob, score;
	double id, id_rev, avg_id;
	double deltaSCI, deltaMeanMFE, deltaConsMFE, deltaZ;
	double consFreqGU;
	
	char outOfRange[1024];
	
	/* neccessary for base pair probability computation */
	double frq[NBASES][NBASES];
	enum base2no {A=1, U, G, C};
	enum base2no base;
	
	/* parse command line */
	if(argc == 1) exit_with_help();
	for(i=1;i<argc;i++){
	  if(argv[i][0] == '-'){
	    switch ( argv[i][1] ){
	    case 'f': 
	      r=sscanf(argv[++i], "%d", &from);
	      if (r!=1) exit_with_help();
	      from_rev=from;
	      break;
	    case 't':
	      r=sscanf(argv[++i], "%d", &to);
	      if (r!=1) exit_with_help();
	      to_rev=to;
	      break;
	    case 's':
	      r=sscanf(argv[++i], "%f", &cutoff);
	      if (r!=1) exit_with_help();
	      break;
	    case 'c':
	      doClassification=0;
	      r=sscanf(argv[++i], "%d", &trainType);
	      if (r!=1) exit_with_help();
	      break;
	    case 'm':
	      modelDir = argv[++i];
	      break;
	    case 'a':
	      mafFile = 1;
	      break;
            /* Not needed as we do not have a model trained for other descriptor combinations. */  
	    /*case 'o': 
	      r=sscanf(argv[++i], "%d", &code);
	      if (r!=1) exit_with_help();
	      if(code != CODE_SCI && 
		 code!= CODE_MEANMFE && 
		 code != CODE_CONSMFE && 
		 code != CODE_Z && 
		 code != CODE_SCI_Z && 
		 code != CODE_SCI_Z_MEANMFE &&
		 code != CODE_SCI_Z_CONSMFE &&
		 code != CODE_SCI_Z_MEANMFE_CONSMFE &&
		 code != CODE_MEANMFE_CONSMFE &&
		 code != CODE_Z_MEANMFE_CONSMFE &&
		 code != CODE_Z_CONSMFE &&
		 code != CODE_SCI_CONSMFE && 
		 code != CODE_SCI_MEANMFE && 
		 code != CODE_SCI_MEANMFE_CONSMFE && 
		 code != CODE_Z_MEANMFE) exit_with_help();
	      break;*/
	    case 'i':
	      strcpy(input,argv[++i]);
	      clust_file = fopen(argv[i], "r");
	      if(clust_file == NULL) {
		fprintf(stderr, "ERROR: Can't open %s\n", argv[i]);
		exit_with_help();
	      }
	      break;
	    case 'r':
	      strcpy(input_rev,argv[++i]);
	      clust_file_rev = fopen(argv[i], "r");
	      if(clust_file_rev == NULL) {
		fprintf(stderr, "ERROR: Can't open %s\n", argv[i]);
		exit_with_help();
	      }
	      break;
	    case 'v':
	      showVersion=1;
	      break;
	    case 'h':
	      showHelp=1; printCode=0;   /* Change printCode=0 for public use!!!*/
	      break;
	    default: exit_with_help();
	    }
	  }
	}
	if (showVersion==1) version();
	if (showHelp==1) exit_with_help();

 	/* Check for models */
	if (modelDir==NULL){   /* Models in default installation directory, if models directory is not given with command line argument -m ? */
		modelDir = (char *) space((unsigned) 100);
	       	strcat(modelDir, "/usr/local/share/RNAstrand/models");	
		DIR *dir = opendir((const char*) modelDir);
		if(dir==(DIR*)NULL){
			modelDir = (char*)NULL;
		}
		closedir(dir);
	}
	if (modelDir==NULL){   /* Models given via RNASTRANDDIR environment variable ?*/
               	modelDir = (char*)getenv("RNASTRANDDIR");
		/* Check if directory exists */
		DIR *dir = opendir((const char*) modelDir);
                if(dir==(DIR*)NULL){ /* No models found. */
	  		fprintf(stderr, "ERROR: Could not find models in directory '%s'\n. You have to set the RNASTRANDDIR "
					"enviroment variable pointing to the model files!\n", modelDir);
			exit(EXIT_FAILURE);
		}
		closedir(dir);
	}
	/* If no models found, user has to set RNASTRANDDIR.*/ 
	strand_env_variable_init("RNASTRANDDIR");

	/* read input clustal file and save aligned sequences in 'AS' and the 
	   corresponding names in 'names' */
	if(clust_file==NULL){
	   nrerror("ERROR: No input alignment given!\n");
	   exit(EXIT_FAILURE);
	}
	
	if(mafFile==0){
	  n_seq = read_clustal(clust_file, AS, names);
	}
	else{
	  n_seq = read_maf(clust_file, AS, names);
	}
	fclose(clust_file);
	if (n_seq==0) nrerror("ERROR: There were problems reading the input file.\n");
	length = (int) strlen(AS[0]);
	totallength = length;

	/* if a slice is specified */
	if (from!=-1 || to!=-1){
	  if ((from!=-1) && (to==-1) || (from==-1) && (to!=-1)){
	    nrerror("ERROR: You have to set both -f and -t parameters"
		    " to score a slice of the alignment.\n");
	  }
	  if ((from>=to)||(from<=0)||(to>length)){
	    nrerror("ERROR: -f and/or -t parameters out of range"
		    " (no reasonable slice specified)\n");
	  }
	  sliceAln((const char **)AS, (char **)window, from, to);
	  if(clust_file_rev == NULL){ sliceAln((const char **)AS, (char **)window_rev, from, to); }
	  length=to-from+1;
	} 
	/* take complete alignment */
	else { 
	  from=1;
	  to=length;
	  sliceAln((const char **)AS, (char **)window, from, length);
	  if(clust_file_rev == NULL){ sliceAln((const char **)AS, (char **)window_rev, from, length); }
	}

	/* Get reverse complement of window */
	if(clust_file_rev == NULL){ revAln((char **)window_rev); length_rev=length; }
	
	/* Reverse complementary alignment given */
	if(clust_file_rev != NULL){
	  if(mafFile==0){
	    n_seq_rev = read_clustal(clust_file_rev, AS_rev, names_rev);
	  }
	  else{
	    n_seq_rev = read_maf(clust_file_rev, AS_rev, names_rev);
	  }
	  fclose(clust_file_rev);
	  if (n_seq_rev==0) nrerror("ERROR: There were problems reading the reverse complementary input file.\n");
	  length_rev = (int) strlen(AS_rev[0]);
	  if(n_seq != n_seq_rev){
	    nrerror("ERROR: Input alignments have different number of sequences!\n");
	    exit(EXIT_FAILURE);
	  }
	  
	  /* if a slice is specified */
	  if (from_rev!=-1 || to_rev!=-1){
	    int t = from_rev;
	    from_rev = length_rev-to_rev+1;
	    to_rev = length_rev-t+1;
	    sliceAln((const char **)AS_rev, (char **)window_rev, from_rev, to_rev);
	  }
	  /* take complete alignment */
	  else { 
	    from_rev = 1;
	    to_rev = length_rev;
	    sliceAln((const char **)AS_rev, (char **)window_rev, from_rev, length_rev);
	  }
	}
	// Mean pairwise identity of forward window and reverse window
	id=meanPairID((const char **)window);
	id_rev=meanPairID((const char **)window_rev);
	avg_id=((double)(id+id_rev))/((double)2.0);

	get_strand_predictors(	(char **)window, (char **)window_rev, (char**)names, 
				length, length_rev, n_seq, &deltaSCI, &deltaMeanMFE, &deltaConsMFE, &deltaZ, &consFreqGU, modelDir);

	if(doClassification == 1) 	// should be uncommented at the end (classification should default)
	{
	    /* do classification */
	    strand(deltaSCI, deltaMeanMFE, deltaConsMFE, deltaZ, n_seq, avg_id, consFreqGU, &score, &prob, &decValue, modelDir, code);
	    
	    printf("\n");
	    printf("***************************** RNAstrand %s ***************************** \n", versionString);
	    printf("\n   Alignment information				      \n");
	    printf("   ---------------------                                      \n\n");
	    printf("   Input alignment:                                   %s\n", input);
	    if(clust_file_rev==NULL){
		printf("   Reverse complementary alignment:                   not specified\n");
	    }
	    else{
	    	printf("   Reverse complementary alignment:                   %s\n", input_rev);
	    }
	    printf("   Model directory:                                   %s\n", modelDir);
	    printf("   Sequences:                                         %u\n", n_seq); 
	    printf("   Slice:                                             %u to %u\n", from,to); 
	    printf("   Columns:                                           %u\n", length);
	    printf("   Average mean pairwise identity:                    %.2f\n", avg_id);
	    printf("   Sum of GU base pair frequency in consensus * 100:  %.2f\n", consFreqGU);
	    printf("\n   Classification variables          %-7s     %-7s     %-7s          \n", "+", "-", "delta");
	    printf("   ------------------------                                             \n\n");
	    //printf("   Code: "); code2string(code); printf("\n");
	    printf("   Structure conservation index:     %-7.2f     %-7.2f     %-7.2f\n", get_strand_forward_SCI(), get_strand_reverse_SCI(), deltaSCI); 
	    printf("   Mean single sequence MFE:         %-7.2f     %-7.2f     %-7.2f\n", get_strand_forward_meanMFE(), get_strand_reverse_meanMFE(), deltaMeanMFE); 
	    printf("   Consensus MFE:                    %-7.2f     %-7.2f     %-7.2f\n", get_strand_forward_consMFE(), get_strand_reverse_consMFE(), deltaConsMFE); 
	    printf("   Mean z-score:                     %-7.2f     %-7.2f     %-7.2f\n", get_strand_forward_Z(), get_strand_reverse_Z(), deltaZ);
	    printf("\n   Prediction                                      \n");
	    printf("   ----------                                      \n\n");
	    printf("   SVM decision value:               %.6f\n", (float)decValue);
	    printf("   SVM strand-class probability:     %.6f\n", (float)prob);
	    printf("   RNAstrand score:                  %.6f\n", (float)score);
	    printf("   Threshold value:	             %.4f\n", (float)cutoff);
	    if (score > cutoff){
	      printf("   Strand prediction:                +\n");
	    }
	    else if(score < (cutoff*(-1.))){
	      printf("   Strand prediction:                -\n");
	    }
	    else{
	      printf("   Strand prediction:                no decision\n");
	    }
	    printf("\n");
	    if(strcmp(get_strand_warning(), "") != 0){
              printf("   WARNING: %s\n\n", get_strand_warning());
	    }
	    printf("**************************************************************************** \n\n");
	    printf("%s",get_strand_output());   // Print sequences and structures
	}
	else{
	  printVector(trainType, deltaSCI, deltaMeanMFE, deltaConsMFE, deltaZ, n_seq, avg_id, consFreqGU);	
	} 

	free_strand();  // Free dynamic global variables in strand.c
	freeAln((char **)AS);
	freeAln((char **)AS_rev);
	freeAln((char **)window);
	freeAln((char **)window_rev);
	freeAln((char **)names);
	freeAln((char **)names_rev);

	return 0;
}

static void exit_with_help() {
	printf("\nUsage:\tRNAstrand [-v] [-h] [-a] [-f from] [-t to] [-s threshold] [-i alignment] [-r reverse complementary alignment]\n\n"
	       "-i      Input alignment - either in CLUSTALW (default) or MAF format (-a option)\n"
	       "-r      Reverse complementary alignment (optional)\n"
	       "        Use 'utils/rcAln.pl' to get a CLUSTALW-realigned reverse complement.\n"
               "        rcAln.pl assumes that clustalw is installed on your system.\n"
	       "-f -t   Score subregion from-to of alignment\n"
               "-s      Threshold of RNAstrand score (default=0.0001)\n"
	       "-a      Alignment in MAF format. If '-a' is missing: alignment in CLUSTALW format\n"
	       //"-m      Specifies model directory if it is different to RNASTRANDDIR\n"
               //"-o      Code which classification variable to use [0-14]\n" 	
	       "-v      Show version information\n"
	       "-h      Show this help message\n\n");

	// Print mapping of codes?
	if(printCode){
	  printf("Code map\n--------\n");
	  code2string(CODE_SCI); printf("\n");
	  code2string(CODE_MEANMFE); printf("\n");
	  code2string(CODE_CONSMFE); printf("\n");
	  code2string(CODE_Z); printf("\n");
	  code2string(CODE_SCI_Z); printf("\n");
	  code2string(CODE_SCI_Z_MEANMFE); printf("\n");
	  code2string(CODE_SCI_Z_CONSMFE); printf("\n");
	  code2string(CODE_SCI_Z_MEANMFE_CONSMFE); printf("\n");
	  code2string(CODE_MEANMFE_CONSMFE); printf("\n");
	  code2string(CODE_Z_MEANMFE_CONSMFE); printf("\n");
	  code2string(CODE_Z_CONSMFE); printf("\n");
	  code2string(CODE_SCI_CONSMFE); printf("\n");
	  code2string(CODE_SCI_MEANMFE); printf("\n"); 
	  code2string(CODE_SCI_MEANMFE_CONSMFE); printf("\n"); 
	  code2string(CODE_Z_MEANMFE); printf("\n");
	  printf("\n\n");
	}

	exit(EXIT_SUCCESS);
}


static void version(){
  printf("\nRNAstrand %s, May 2007\n\n", versionString);
  exit(EXIT_SUCCESS);
}


/*****************************
* code taken from RNAz 0.1.1 
*****************************/
static int read_clustal(FILE *clust, char *alignedSeqs[], char *names[]) {

  char *line, name[100]={'\0'}, *seq;
  int  n, nn=0, num_seq = 0;
   
  if ((line=(char *)get_line(clust)) == NULL) {
	fprintf(stderr, "ERROR: Empty CLUSTAL file\n"); return 0;
  }

  if (strncmp(line,"CLUSTAL", 7) !=0) {
	fprintf(stderr, "ERROR: No CLUSTAL file\n");
	free(line); return 0;
  }
  free(line);
  line = (char *)get_line(clust);

  while (line!=NULL) {
	if (((n=strlen(line))<4) || isspace((int)line[0])) {
	  /* skip non-sequence line */
	  free(line); line = (char *)get_line(clust);
	  nn=0; /* reset seqence number */
	  continue;
	} 
     
	seq = (char *) space( (n+1)*sizeof(char) );
	sscanf(line,"%99s %s", name, seq);
	if (nn == num_seq) { /* first time */
	  names[nn] = strdup(name);
	  alignedSeqs[nn] = strdup(seq);
	}
	else {
	  if (strcmp(name, names[nn])!=0) {
		/* name doesn't match */
		fprintf(stderr,
				"ERROR: Inconsitent sequence names in CLUSTAL file)\n");
		free(line); free(seq);
		return 0;
	  }
	  alignedSeqs[nn] = (char *)
		xrealloc(alignedSeqs[nn], strlen(seq)+strlen(alignedSeqs[nn])+1);
	  strcat(alignedSeqs[nn], seq);
	}
	nn++;
	if (nn>num_seq) num_seq = nn;
	free(seq);
	free(line);
	if (num_seq>=MAX_NUM_NAMES) {
	  fprintf(stderr, "ERROR: Too many sequences in CLUSTAL file\n");
	  return 0;
	}
	
	line = (char *)get_line(clust);
  }
   
  alignedSeqs[num_seq] = NULL;
  if (num_seq == 0) {
	fprintf(stderr, "ERROR: No sequences found in CLUSTAL file\n");
	return 0;
  }
  n = strlen(alignedSeqs[0]); 
  for (nn=1; nn<num_seq; nn++) {
	if (strlen(alignedSeqs[nn])!=n) {
	  fprintf(stderr, "ERROR: Sequences are of unequal length.\n");
	  return 0;
	}
  }
  return num_seq;
}


static int read_maf(FILE *clust, char *alignedSeqs[], char *names[]) {

  char *line, name[100]={'\0'}, *seq, *t_str;
  int  n, nn=0, num_seq = 0, t_int1, t_int2, t_int3;
   
  if ((line=(char *)get_line(clust)) == NULL) {
	fprintf(stderr, "ERROR: Empty MAF file\n"); return 0;
  }

  if (strncmp(line,"##maf", 5) !=0) {
	fprintf(stderr, "ERROR: No MAF file\n");
	free(line); return 0;
  }
  free(line);
  line = (char *)get_line(clust);

  while (line!=NULL) {
    n=strlen(line);

    if ( n<4 || isspace((int)line[0]) || strncmp(line, "s",1)!=0 ) {
      /* skip non-sequence line */
      free(line); line = (char *)get_line(clust);
      nn=0; /* reset seqence number */
      continue;
    } 

    if(strncmp(line,"s",1) == 0){ /* Sequence line */

      seq = (char *) space( (n+1)*sizeof(char) );
      t_str = (char *) space( (2)*sizeof(char) );
      if(sscanf(line,"s %s %d %d %s %d %s", name, &t_int1, &t_int2, t_str, &t_int3, seq) == 6){
	if (nn == num_seq) { /* first time */
	  names[nn] = strdup(name);
	  alignedSeqs[nn] = strdup(seq);
	}
	else {
	  if (strcmp(name, names[nn])!=0) {
	    /* name doesn't match */
	    fprintf(stderr,
		    "ERROR: Inconsistent sequence names in MAF file)\n");
	    free(line); free(seq);
	    return 0;
	  }
	  alignedSeqs[nn] = (char *)
	    xrealloc(alignedSeqs[nn], strlen(seq)+strlen(alignedSeqs[nn])+1);
	  strcat(alignedSeqs[nn], seq);
	}
	nn++;
	if (nn>num_seq) num_seq = nn;
	free(seq);
	free(line);
	if (num_seq>=MAX_NUM_NAMES) {
	  fprintf(stderr, "ERROR: Too many sequences in MAF file\n");
	  return 0;
	}
      }
      else{
	fprintf(stderr, "ERROR: Parsing of sequence lines failed!\n");
	return 0;
      }
    }
    line = (char *)get_line(clust);
  }
   
  alignedSeqs[num_seq] = NULL;
  if (num_seq == 0) {
    fprintf(stderr, "ERROR: No sequences found in CLUSTAL file\n");
    return 0;
  }
  n = strlen(alignedSeqs[0]); 
  for (nn=1; nn<num_seq; nn++) {
    if (strlen(alignedSeqs[nn])!=n) {
      fprintf(stderr, "ERROR: Sequences are of unequal length.\n");
      return 0;
    }
  }
  return num_seq;
}


/*****************************
* code taken from RNAz 0.1.1 
*****************************/
static void sliceAln(const char *sourceAln[], char *destAln[], int from, int to){

  int i;
  char *slice;

  for (i=0;sourceAln[i]!=NULL;i++){
	slice=(char *) space((unsigned) (to-from+2));
	strncpy(slice,sourceAln[i]+from-1,(to-from+1));
	destAln[i]=slice;
  }
}

/**************************************
* original code taken from RNAz 0.1.1 
**************************************/
static void revAln(char *AS[]) {

  int i,j,length;
  char *tmp;
  char letter;
  length=strlen(AS[0]);
  
  for (i=0;AS[i]!=NULL;i++){
	tmp = (char *) space((unsigned) length+1);
	for (j=length-1;j>=0;j--){
	  letter=AS[i][j];
	  switch(letter){
		case 'T': letter='A'; break;
		case 'U': letter='A'; break;
		case 'C': letter='G'; break;
		case 'G': letter='C'; break;
		case 'A': letter='U'; break;
		/* N, S and W have itself as complement */
		case 'V': letter='B'; break;
		case 'B': letter='V'; break;
		case 'H': letter='D'; break;
		case 'D': letter='H'; break; 
		case 'K': letter='M'; break;
		case 'M': letter='K'; break;
		case 'Y': letter='R'; break;
		case 'R': letter='Y'; break;
	  }
	  tmp[length-j-1]=letter;
	}
	tmp[length]='\0';
	strcpy(AS[i],tmp);
	free(tmp);
	tmp=NULL;
  }
}

/*****************************
* code taken from RNAz 0.1.1 
*****************************/
static void freeAln(char *AS[]){

  int i;
  for (i=0;AS[i]!=NULL;i++){
        free(AS[i]);
  }
  AS=NULL;
}

static void printVector(int trainType, double deltaSCI, double deltaMeanMFE, double deltaConsMFE, double deltaZ_score, int n_seq, double id, double consFreqGU) 
{
  if(trainType != 1 && trainType != -1) exit_with_help();
  printf("%d", trainType);	
  printf(" %.2lf", deltaSCI);
  printf(" %.2lf", deltaMeanMFE);
  printf(" %.2lf", deltaConsMFE);
  printf(" %.2lf", deltaZ_score);
  printf(" %d", n_seq);
  printf(" %.2lf", id);
  printf(" %.2lf", consFreqGU);
  printf("\n");
  
  return;
}

static void printDecVector(int dec, double prob, double decV, double deltaSCI, double deltaMeanMFE, double deltaConsMFE, double deltaZ_score, 
		    int ai, char * i, int f, int t, int l) {
  printf("%d", dec);
  printf(" %.6f", (float)prob);
  printf(" %.2f", (float)decV);
  printf(" 1:%.2f", (float)deltaSCI);
  printf(" 2:%.2f", (float)deltaMeanMFE);
  printf(" 3:%.2f", (float)deltaConsMFE);
  printf(" 4:%.2f", (float)deltaZ_score);
  if(ai == 1) printf(" %s", i);
  printf(" from:%d to:%d (total:%d)",f,t,l);
  printf("\n");
  
  return;
}

/*****************************
* code taken from RNAz 0.1.1 
*****************************/
static double meanPairID(const char *AS[]) {

  int i,j,k,matches,pairs,length;

  matches=0;
  pairs=0;

  length=strlen(AS[0]);
  
  for (i=0;AS[i]!=NULL;i++){
	for (j=i+1;AS[j]!=NULL;j++){
	  for (k=0;k<length;k++){
		if ((AS[i][k]!='-') || (AS[j][k]!='-')){
		  if (AS[i][k]==AS[j][k]){
			matches++;
		  }
		  pairs++;
		}
	  }
	}
  }

  return (double)(matches)/pairs*100;
  
}
