#!/usr/bin/perl -w

use strict;
use Getopt::Long;
use Pod::Usage;

main();

sub main
{
	my $input = '';
	my $output = '';
	my %hSeq = ();
	my $help = 0;

	if($#ARGV<0){ pod2usage(-verbose => 1); }
	GetOptions('input:s' => \$input,
                   'i:s' => \$input,
                   'output:s' => \$output,
                   'o:s' => \$output,
                   'help'=>\$help,
                   'h'=>\$help,
                  ) or pod2usage(-verbose => 1);
	if($help){ pod2usage(-verbose => 2); }

	open(IN, "$input") or die "ERROR ($0): No input alignment '$input'.\n";
	my @aln = <IN>;
	close IN;
	read_clustalw(\%hSeq, \@aln);
	reverse_complement(\%hSeq);
	if($output=~/\.aln$/){
                $output=~s/\.aln/\.fa/;
        }	
	else{
		$output.="\.aln";
	}
	print_fasta(\%hSeq, $output);
	call_clustalw($output);
	exit(1);
}

#################################################################
# Subroutines
#################################################################

sub reverse_complement
{
	my ($refHSeq) = @_;

	my $l = 0;
        my $oldl = 0;
        my $first=1;
        foreach my $k(keys %{$refHSeq}){
                $refHSeq->{$k} = complement($refHSeq->{$k});
                $refHSeq->{$k} = reverse($refHSeq->{$k});
                $l = length($refHSeq->{$k});
                if($l!=$oldl && !$first){
                        die "ERROR ($0): Sequences have different length!\n";
                }
        	$first=0;
        	$oldl = $l;
        }
	return;	
}

sub read_clustalw
{
	my ($refHSeq, $refAln) = @_;

	foreach my $line(@{$refAln}){
    		chomp $line;
    		if($line=~/[Cc][Ll][Uu][Ss][Tt][Aa][Ll]/){
			next;
		}
		elsif($line=~/^[\s\*]+$/){ next; }
    		elsif($line=~/^\s*(\S+)\s+(\S+)\s*$/){
			my $name = $1;
			my $subseq = $2;
			if(!exists $refHSeq->{$name}){
				$refHSeq->{$name}="";
			}
			$refHSeq->{$name} .= $subseq;
    		}
	}
	return;
}

sub print_fasta
{
	my ($refHSeq, $file) = @_;
	
	# Print FASTA file
	open(FASTA, ">$file") or die "ERROR ($0): Could not open file '$file'.\n";
	foreach my $k(keys %{$refHSeq}){
		if(!exists $refHSeq->{$k}){ die "ERROR ($0): No sequence with name '$k' given.\n "; }
	        print FASTA ">$k\n$refHSeq->{$k}\n";
	}
	close FASTA;
	return;
}

sub call_clustalw
{
	my ($f)=@_;
        my $sysCommand = "clustalw $f";
        system($sysCommand)==0 or die "ERROR ($0): No clustalw alignment of reverse complementary sequences.\n";
	return;
}

sub complement
{
	my ($seq) = shift;
	$seq =~ tr/ACGTUNVBHDKMSWYR/UGCAANBVDHMKSWRY/;
    	$seq =~ tr/acgtunvbhdkmswyr/ugcaanbvdhmkswry/;
    	return $seq;
}

#################################################################
# Documentation
#################################################################

__END__

=head1 NAME

<rcAln.pl> - Creates reverse complementary alignment by calling clustalw.
             Hence, clustalw must be installed on the user system.

=head1 SYNOPSIS

 rcAln.pl [options]

=head1 OPTIONS

=over 3

=item B<-i, --input>

Input alignment in ClustalW format.

=item B<-o, --output>

Name of output file the reverse complementary alignment is written to. Name should be given as "name.aln" with suffix ".aln". 
If suffix is missing "name" is extended to "name.aln". The reverse complementary sequences are written to "name.fa" and the tree to "name.dnd".  

=item B<-h, --help>

Prints manual and exits.

=back

=head1 EXAMPLES

 # rcAln.pl --input input.aln --output output.aln

=head1 AUTHORS

Kristin Reiche <kristin@bioinf.uni-leipzig.de>

=cut

